# -*- coding: utf-8 -*-
"""
    API - Strawberry Types
"""


import dataclasses as dc
import datetime
import decimal
import typing

import fastberry

# Model
model = fastberry.Model()

# Create your <types> here.



"""
import motor.motor_asyncio

# Config
DATABASE_URL = "mongodb://localhost:27017"
DATABASE_NAME = "test_database"

# Engine
ENGINE = motor.motor_asyncio.AsyncIOMotorClient(DATABASE_URL)


# Base
Base = ENGINE[DATABASE_NAME]
"""



#!/usr/bin/env python
"""
Project Manager
"""

import pathlib

from fastberry import Fastberry

BASE_DIR = pathlib.Path(__file__).resolve().parents[0]

manager = Fastberry(base_dir=BASE_DIR, is_cli=True)

if __name__ == "__main__":
    manager.cli()



"""Test - Library
"""

import os
import pathlib
import sys


def dir_up(depth):
    """Easy level-up folder(s)."""
    return sys.path.append(os.path.join(pathlib.Path(__file__).parents[depth], "src"))


# Append to (sys.path)
dir_up(1)


# Test
import fastberry

print(dir(fastberry))



# -*- coding: utf-8 -*-
"""
    API - CRUD
"""

# Fastberry
from fastberry import GQL


# Create your API (GraphQL) here.
class Demo(GQL):
    """Demo Api"""

    schema = None
    prefix = "demo"

    class Query:
        """Query"""

        async def search(info) -> str:
            """Read the Docs"""
            print(info)
            return "Search"

        async def detail(info) -> str:
            """Read the Docs"""
            print(info)
            return "Detail"

    class Mutation:
        """Mutation"""

        async def create(info) -> str:
            """Read the Docs"""
            print(info)
            return "Create"

        async def update(info) -> str:
            """Read the Docs"""
            print(info)
            return "Update"

        async def delete(info) -> str:
            """Read the Docs"""
            print(info)
            return "Delete"



# -*- coding: utf-8 -*-
""" [Main]
    FastAPI Main File.
"""

from fastapi import FastAPI
from fastapi.middleware.cors import CORSMiddleware
from fastapi.middleware.gzip import GZipMiddleware
from fastapi.responses import RedirectResponse
from strawberry.fastapi import GraphQLRouter

# API (Config)
from config import settings

# API (Schema)
from fastberry import Schema

# FastAPI
app = FastAPI(
    description=settings.docs,
    title=settings.base.app_name,
    version=settings.base.version,
)

# Middleware
app.add_middleware(GZipMiddleware, minimum_size=1000)
settings.middleware(app)

# CORS
if settings.base.allowed_hosts:
    app.add_middleware(
        CORSMiddleware,
        allow_origins=settings.base.allowed_hosts,
        allow_credentials=True,
        allow_methods=["*"],
        allow_headers=["*"],
    )

# ALL Routers
app.include_router(settings.router)

# GraphQL (Schema)
schema = Schema(
    query=settings.apps.schema.Query,
    mutation=settings.apps.schema.Mutation,
    extensions=[],
    introspection=(not settings.mode == "production"),
)

# GraphQL (Router)
if schema:
    app.include_router(
        GraphQLRouter(
            schema,
            graphiql=(not settings.mode == "production"),
        ),
        prefix="/graphql",
        tags=["GraphQL"],
    )

# Startup
@app.on_event("startup")
async def startup_event():
    for func in settings.on_event["startup"]:
        func()


# Shutdown
@app.on_event("shutdown")
def shutdown_event():
    for func in settings.on_event["shutdown"]:
        func()


# Redirect To Docs
@app.get("/", response_class=RedirectResponse)
async def redirect_fastapi():
    return "/docs"
